@@ -1,46 +1,36 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-from abc import ABC, abstractmethod
-import warnings
+
+import builtins
 import collections
 import copy
-import hashlib
-import numbers
 import inspect
 import itertools
-import builtins
-import os
-import pickle
+import numbers
 import random
-import string
 import sys
 import threading
 import time
-from enum import Enum, unique
-from typing import List, TypeVar, Tuple, Union, Optional, Generic, Iterable, Iterator, Sequence, Dict
+from abc import ABC, abstractmethod
+from typing import TypeVar, Tuple, Optional, Iterator, Dict
 
 import cv2
 import numpy as np
-from skimage import color
-from trident.backend.numpy_ops import DTYPE_MAPPING
-
-from trident.backend.opencv_backend import file2array
-from trident.data.vision_transforms import Unnormalize
-
-from trident.data.transform import Transform
-
-from trident.data.bbox_common import xywh2xyxy, xyxy2xywh
-from trident.data.image_common import gray_scale, image2array, mask2array, image_backend_adaption, reverse_image_backend_adaption, \
-    unnormalize, array2image, GetImageMode
 
 from trident.backend import iteration_tools
+from trident.backend.common import *
+from trident.backend.numpy_ops import DTYPE_MAPPING
+from trident.backend.opencv_backend import file2array
+from trident.backend.tensorspec import TensorSpec
+from trident.data.image_common import image_backend_adaption, reverse_image_backend_adaption, \
+    array2image
 from trident.data.label_common import label_backend_adaptive
 from trident.data.mask_common import mask_backend_adaptive, color2label
-from trident.data.text_common import text_backend_adaption, reverse_text_backend_adaption
 from trident.data.samplers import *
-from trident.backend.common import *
-from trident.backend.tensorspec import TensorSpec, assert_input_compatibility
+from trident.data.text_common import reverse_text_backend_adaption
+from trident.data.transform import Transform
+from trident.data.vision_transforms import Unnormalize
 
 try:
     import Queue
@@ -48,12 +38,11 @@ except ImportError:
     import queue as Queue
 
 if get_backend() == 'pytorch':
-    from trident.backend.pytorch_backend import to_numpy, to_tensor, ObjectType
-    from trident.backend.pytorch_ops import int_shape, str2dtype, tensor_to_shape, expand_dims, cast
-    import torch
+    from trident.backend.pytorch_backend import to_tensor, ObjectType
+    from trident.backend.pytorch_ops import tensor_to_shape, expand_dims, cast
 elif get_backend() == 'tensorflow':
-    from trident.backend.tensorflow_backend import to_numpy, to_tensor, ObjectType
-    from trident.backend.tensorflow_ops import int_shape, str2dtype, tensor_to_shape, expand_dims, cast
+    from trident.backend.tensorflow_backend import to_tensor, ObjectType
+    from trident.backend.tensorflow_ops import tensor_to_shape, expand_dims, cast
 
 __all__ = ['Dataset', 'ZipDataset', 'ImageDataset', 'MaskDataset', 'TextSequenceDataset', 'LabelDataset', 'BboxDataset', 'LandmarkDataset', 'Iterator', 'MetricIterator',
            'NumpyDataset', 'RandomNoiseDataset']
@@ -95,10 +84,9 @@ class DatasetBase(ABC):
 class Dataset(DatasetBase):
     def __init__(self, args, symbol=None, object_type: Optional[ObjectType] = None, **kwargs):
         super().__init__()
-        if args is None:
-            self.items = []
-        else:
-            self.items = args
+        self.items = []
+        if args is not None and hasattr(args, '__iter__'):
+            self.items.extend(args)
         self._element_spec = None
 
         if symbol == None:
@@ -115,6 +103,8 @@ class Dataset(DatasetBase):
         self.transform_funcs = []
 
     def __getitem__(self, index: int) -> Tuple:
+        if index >= len(self.items):
+            index = index % len(self.items)
         return self.items[index]
 
     def __setattr__(self, name: str, value) -> None:
@@ -212,7 +202,7 @@ class ZipDataset(Dataset):
         self.symbol = tuple([ds.symbol for ds in datasets])
 
     def __getitem__(self, index: int) -> Tuple:
-        return tuple([ds[index%len(ds)] for ds in self.items])
+        return tuple([ds[index] for ds in self.items])
 
     def __getattr__(self, name):
         if name in self.__dict__:
@@ -247,8 +237,21 @@ class NumpyDataset(Dataset):
                                             object_type=self.object_type)
 
     def __getitem__(self, index: int) -> Tuple:
+        if index >= len(self.items):
+            index = index % len(self.items)
         return None if self.items is None else self.items[index]
 
+    def data_transform(self, img_data):
+
+        if isinstance(img_data, np.ndarray):
+            for fc in self.transform_funcs:
+                if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption:
+                    img_data = fc(img_data, spec=self.element_spec)
+
+            return img_data
+        else:
+            return img_data
+
     def __len__(self) -> int:
         return None if self.items is None else len(self.items)
 
@@ -267,7 +270,9 @@ class StreamDataset(Dataset):
     def __iter__(self):
         pass
 
-    def __getitem__(self):
+    def __getitem__(self,index):
+        if index >= len(self.items):
+            index = index % len(self.items)
         raise AssertionError("can not get item from StreamDataset by index")
 
     def __len__(self):
@@ -275,22 +280,33 @@ class StreamDataset(Dataset):
 
 
 class ArrayDataset(Dataset):
-    def __init__(self, arrays):
+    def __init__(self, arrays, symbol='array'):
         r"""
         ArrayDataset is a dataset for numpy array data, one or more numpy arrays
          are needed to initiate the dataset. And the dimensions represented sample number
          are expected to be the same.
         """
-        super().__init__()
-        if not all(len(arrays) == len(array) for array in arrays):
-            raise ValueError("lengths of input arrays are inconsistent")
-        self.arrays = arrays
+        super().__init__(arrays)
+        self.symbol = symbol
 
-    def __getitem__(self, index: int) -> Tuple:
-        return self.arrays[index]
+    def __getitem__(self, index: int):
+        if index >= len(self.items):
+            index = index % len(self.items)
+        return self.items[index]
 
     def __len__(self) -> int:
-        return len(self.arrays)
+        return len(self.items)
+
+    def data_transform(self, img_data):
+
+        if isinstance(img_data, np.ndarray):
+            for fc in self.transform_funcs:
+                if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption:
+                    img_data = fc(img_data, spec=self.element_spec)
+
+            return img_data
+        else:
+            return img_data
 
 
 class ImageDataset(Dataset):
@@ -300,9 +316,14 @@ class ImageDataset(Dataset):
         self.is_paired_process = False
 
     def __getitem__(self, index: int):
+        if index>=len(self.items):
+            index=index%len(self.items)
         img = self.items[index]  # self.pop(index)
+
         if isinstance(img, str) and self.object_type==ObjectType.image_path:
             return img
+        elif isinstance(img, np.ndarray) and self.object_type!=ObjectType.image_path:
+            return img
         else:
             img = file2array(img)
 
@@ -406,6 +427,8 @@ class MaskDataset(Dataset):
         self._element_spec = TensorSpec(shape=TensorShape(self[0]), name=self.symbol, object_type=self.object_type, is_spatial=True)
 
     def __getitem__(self, index: int):
+        if index >= len(self.items):
+            index = index % len(self.items)
         mask = self.items[index]  # self.pop(index)
 
         if isinstance(mask, str):
@@ -510,6 +533,8 @@ class LabelDataset(Dataset):
             self._idx2lab = dict(zip(range(len(self.class_names[language])), self.class_names[language]))
 
     def __getitem__(self, index: int):
+        if index >= len(self.items):
+            index = index % len(self.items)
         label = self.items[index]
         return label
 
@@ -557,7 +582,8 @@ class BboxDataset(Dataset):
             self._current_idx = -1
 
     def __getitem__(self, index: int):
-        self._current_idx = index
+        if index >= len(self.items):
+            index = index % len(self.items)
         bboxes = self.items[index].astype(np.float32)
         if self.object_type == ObjectType.relative_bbox and (self.image_size is None):
             raise RuntimeError('You need provide image size information for calculate relative_bbox. ')
@@ -597,14 +623,8 @@ class LandmarkDataset(Dataset):
         self.transform_funcs = []
 
     def __getitem__(self, index: int):
-        # if (landmarks > 1).any() or (self.image_size is None):
-        #     return np.array(landmarks).astype(np.float32)
-        # # elif (landmarks > 1).any():
-        # #     height, width = self.image_size
-        # #     landmarks[:, 0] = landmarks[:, 0] / width
-        # #     landmarks[:, 1] = landmarks[:, 1] / height
-        # #     return np.array(landmarks).astype(np.float32)
-        # else:
+        if index >= len(self.items):
+            index = index % len(self.items)
         return self.items[index]
 
     def landmark_transform(self, *landmarks):
@@ -1037,7 +1057,7 @@ class Iterator(object):
         self.paired_process_symbols = []
 
         self.is_shuffle = is_shuffle
-
+        self.memory_cache=[]
 
 
         self.data_template = OrderedDict()
@@ -1092,10 +1112,11 @@ class Iterator(object):
                 if len(ds) > 0:
                     dataitem = ds[k]
                     ds.element_spec = TensorSpec.tensor_to_spec(expand_dims(dataitem, 0), object_type=ds.object_type, name=ds.symbol)
-                    self.data_template[ds.element_spec] = None
+            self.data_template[ds.element_spec] = None
 
         self._batch_size = batch_size
         self.paired_transform_funcs = []
+        self.batch_transform_funcs=[]
 
         self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
@@ -1209,6 +1230,23 @@ class Iterator(object):
                 PrintException()
         return datadict
 
+
+
+    def batch_transform(self,datadict: Dict[TensorSpec, np.ndarray]):
+        if  len(self.batch_transform_funcs)>0:
+            if len(self.batch_transform_funcs) == 0:
+                return datadict
+
+                # if img_data.ndim>=2:
+            for fc in self.batch_transform_funcs:
+                try:
+                    if hasattr(fc,'memory_cache'):
+                        fc.memory_cache=self.memory_cache
+                        datadict = fc(datadict)
+                except:
+                    PrintException()
+            return datadict
+
     def get_datasets(self):
         datasets = []
         if self._data and isinstance(self._data, Dataset) and not isinstance(self._data, ZipDataset) and len(self._data) > 0:
@@ -1274,12 +1312,13 @@ class Iterator(object):
 
             returnData =OrderedDict()# copy.deepcopy(self.data_template)
 
-            data = self.data.__getitem__(index % len(self.data)) if self.data is not None and len(self.data) > 0 else None
-            label = self.label.__getitem__(index % len(self.label)) if self.label is not None and len(self.label) > 0 else None
+            data = self.data.__getitem__(index) if self.data is not None and len(self.data) > 0 else None
+            label = self.label.__getitem__(index) if self.label is not None and len(self.label) > 0 else None
 
-            unpair = self.unpair.__getitem__(index % len(self.unpair)) if self.unpair is not None and len(self.unpair) > 0 else None
+            unpair = self.unpair.__getitem__(index) if self.unpair is not None and len(self.unpair) > 0 else None
             results = iteration_tools.flatten((data, label, unpair), iterable_types=(tuple))
             results = tuple([item for item in results if item is not None])
+
             if len(returnData) > 0 and len(returnData) != len(self.get_datasets()):
                 raise ValueError("Flattened data should have same length as datasets")
 
@@ -1292,6 +1331,17 @@ class Iterator(object):
             if len(self.paired_process_symbols) > 0:
                 returnData = self.paired_transform(returnData)
 
+
+            if len(self.batch_transform_funcs) > 0:
+                self.memory_cache.append(copy.deepcopy(returnData))
+                if len(self.memory_cache)>self._batch_size:
+                    self.memory_cache.pop(0)
+                if  len(self.memory_cache)>self._batch_size//2:
+                    returnData = self.batch_transform(returnData)
+
+            # for i in range(len(returnData)):
+            #     ds = self.get_datasets()[i]
+            #     returnData[ds.element_spec] = unpack_singleton(ds.data_transform(returnData.value_list[i]))
             def process_data_transform(i):
                 ds = self.get_datasets()[i]
                 returnData[ds.element_spec] = unpack_singleton(ds.data_transform(returnData.value_list[i]))
@@ -1304,10 +1354,10 @@ class Iterator(object):
                 threads[i].join()
 
 
-            if self.signature is None or len(self.signature) == 0:
-                self.signature = Signature(name='data_provider')
+            if self.signature is None or len(self.signature.outputs) == 0:
+                self._signature = Signature(name='data_provider')
                 for spec in returnData.key_list:
-                    self.signature.outputs[spec.name] = spec
+                    self._signature.outputs[spec.name] = spec
 
             self.pass_cnt += 1
             self.pass_time_spend += float(time.time() - start_time)
