@@ -696,8 +696,8 @@ class RandomTransformAffine(VisionTransform):
         if self.shift_range is not None:
             if isinstance(self.shift_range, numbers.Number):
                 self.shift_range = (-1 * self.shift_range, self.shift_range)
-            max_dx = self.shift_range[0] * image.shape[0]
-            max_dy = self.shift_range[1] * image.shape[1]
+            max_dx = self.shift_range[0] * image.shape[1]
+            max_dy = self.shift_range[1] * image.shape[0]
             translations = (np.round(random.uniform(-max_dx, max_dx)),
                             np.round(random.uniform(-max_dy, max_dy)))
         else:
@@ -721,8 +721,8 @@ class RandomTransformAffine(VisionTransform):
         else:
             shear = 0.0
 
-        self.output_size = image.shape[0]
-        center = (image.shape[0] * 0.5 + 0.5, image.shape[1] * 0.5 + 0.5)
+
+        center = (image.shape[1] * 0.5 + 0.5, image.shape[0] * 0.5 + 0.5)
 
         angle = math.radians(angle)
         # if isinstance(shear, (tuple, list)) and len(shear) == 2:
@@ -1029,8 +1029,12 @@ class AdjustContrast(VisionTransform):
         if self.value == 0:
             return image
         image = image.astype(np.float32)
-        mean = round(cv2.cvtColor(image, cv2.COLOR_RGB2GRAY).mean())
-        image = (1 - self.value) * mean + self.value * image
+        mean_value=-1
+        if ndim(image)==2 or (ndim(image)==3 and image.shape[-1]==1):
+            mean_value=image.mean()
+        else:
+            mean_value = round(cv2.cvtColor(image, cv2.COLOR_RGB2GRAY).mean())
+        image = (1 - self.value) * mean_value + self.value * image
         return image.clip(0, 255).astype(np.float32)
 
     def _apply_coords(self, coords, spec: TensorSpec):
@@ -1708,7 +1712,6 @@ class RandomErasing(VisionTransform):
     def _apply_mask(self, mask, spec: TensorSpec):
         return mask
 
-
 class ImageMosaic(VisionTransform):
     """Perform gamma correction on an image.
         Also known as Power Law Transform. Intensities in RGB mode are adjusted
@@ -1726,143 +1729,274 @@ class ImageMosaic(VisionTransform):
         super().__init__(name)
         self.output_size = output_size
         self.keep_prob = keep_prob
+        self.memory_cache=[]
 
-    def apply_batch(self, inputs: Sequence[Tuple], spec: Optional[TensorSpec] = None):
-        r"""Apply transform on batch input data."""
-        if not isinstance(inputs, OrderedDict):
-            if spec is None and self.is_spatial == True:
-                self._shape_info = None
-                spec = TensorSpec(shape=tensor_to_shape(inputs, need_exclude_batch_axis=True, is_singleton=True), object_type=ObjectType.rgb)
-            return self.apply(inputs, spec)
-        else:
-            results = OrderedDict()
-            imgs = None
-            boxes = None
-            masks = None
-            imgs = inputs.value_list[0]
-
-            if int_shape(imgs)[0] <= 4:
-                return inputs
-            else:
-                for i in range(len(inputs.values())):
-                    k, v = inputs.item_list[i]
-                    results[k] = []
-                    if i == 0:
-                        imgs = v
-                    elif isinstance(k, TensorSpec) and 'box' in str(k.object_type).lower():
-                        boxes = v
-                    elif isinstance(k, TensorSpec) and 'mask' in str(k.object_type).lower():
-                        masks = v
-                if get_backend() == 'pytorch':
-                    imgs = imgs.transpose([0, 2, 3, 1])
-
-                for i in range(int_shape(imgs)[0]):
-                    rr = random.random()
-                    if rr > self.keep_prob:
-
-                        img = imgs[i]
-
-                        other_idxes = list(range(int_shape(imgs)[0]))
-                        other_idxes.remove(i)
-                        random.shuffle(other_idxes)
-                        other_idxes = other_idxes[:3]
-
-                        height, width = self.output_size
-                        rs = np.random.uniform(0.5, 1.5, [2])  # random shift
-                        center = (int(height * rs[0] / 2), int(width * rs[1] / 2))
-
-                        # crop each image
-                        p1 = imgs[other_idxes[0]].copy()[:center[0], center[1]:width, :]
-                        p2 = imgs[other_idxes[1]].copy()[center[0]:height, :center[1], :]
-                        p3 = imgs[other_idxes[2]].copy()[center[0]:height, center[1]:width, :]
-
-                        base_img = img.copy()
-                        base_img[:center[0], center[1]:width, :] = p1
-                        base_img[center[0]:height, :center[1], :] = p2
-                        base_img[center[0]:height, center[1]:width, :] = p3
-
-                        results[results.key_list[0]].append(base_img)
-
-                        if masks is not None and masks[i].any():
-                            msk = masks[i]
-                            mp1 = masks[other_idxes[0]].copy()[:center[0], center[1]:width]
-                            mp2 = masks[other_idxes[1]].copy()[center[0]:height, :center[1]]
-                            mp3 = masks[other_idxes[2]].copy()[center[0]:height, center[1]:width]
-
-                            base_msk = msk.copy()
-                            base_msk[:center[0], center[1]:width] = mp1
-                            base_msk[center[0]:height, :center[1]] = mp2
-                            base_msk[center[0]:height, center[1]:width] = mp3
-
-                            mask_key = [k for k in results.key_list if 'mask' in str(k.object_type).lower()]
-                            if mask_key:
-                                results[mask_key[0]].append(base_msk)
-
-                        mosaic_bboxes = []
-                        if boxes is not None and boxes[i].any():
-                            box = boxes[i]
-                            box1 = boxes[other_idxes[0]]
-                            box2 = boxes[other_idxes[1]]
-                            box3 = boxes[other_idxes[2]]
-                            box_list = []
-                            if box.any():
-                                box[:, 0][box[:, 0] > center[1]] = center[1]
-                                box[:, 2][box[:, 2] > center[1]] = center[1]
-                                box[:, 1][box[:, 1] > center[0]] = center[0]
-                                box[:, 3][box[:, 3] > center[0]] = center[0]
-                                box_list.append(box)
-                            if box1.any():
-                                box1[:, 0][box1[:, 0] < center[1]] = center[1]
-                                box1[:, 2][box1[:, 2] < center[1]] = center[1]
-                                box1[:, 1][box1[:, 1] > center[0]] = center[0]
-                                box1[:, 3][box1[:, 3] > center[0]] = center[0]
-                                box_list.append(box1)
-
-                            if box2.any():
-                                box2[:, 0][box2[:, 0] > center[1]] = center[1]
-                                box2[:, 2][box2[:, 2] > center[1]] = center[1]
-                                box2[:, 1][box2[:, 1] < center[0]] = center[0]
-                                box2[:, 3][box2[:, 3] < center[0]] = center[0]
-                                box_list.append(box2)
-
-                            if box3.any():
-                                box3[:, 0][box3[:, 0] < center[1]] = center[1]
-                                box3[:, 2][box3[:, 2] < center[1]] = center[1]
-                                box3[:, 1][box3[:, 1] < center[0]] = center[0]
-                                box3[:, 3][box3[:, 3] < center[0]] = center[0]
-                                box_list.append(box3)
-
-                            if len(box_list) > 0:
-                                mosaic_bboxes = np.concatenate(box_list)
-                                # remove no area bbox
-                                keep_x = mosaic_bboxes[:, 2] - mosaic_bboxes[:, 0] > 16
-                                keep_y = mosaic_bboxes[:, 3] - mosaic_bboxes[:, 1] > 16
-                                keep_mask = np.logical_and(keep_x, keep_y)
-                                mosaic_bboxes = mosaic_bboxes[keep_mask]
-                            else:
-                                mosaic_bboxes = list()
-                        box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
-                        if box_key:
-                            results[box_key[0]].append(mosaic_bboxes)
-                    else:
-                        results[results.key_list[0]].append(imgs[i])
-                        mask_key = [k for k in results.key_list if 'mask' in str(k.object_type).lower()]
-                        if mask_key:
-                            results[mask_key[0]].append(masks[i])
-                        box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
-                        if box_key:
-                            results[box_key[0]].append(boxes[i])
-
-            for i in range(len(results)):
-                if i == 0:
-                    result_imgs = np.stack(results.value_list[i], 0)
-                    if get_backend() == 'pytorch':
-                        result_imgs = result_imgs.transpose([0, 3, 1, 2])
-
-                    results[results.key_list[0]] = result_imgs
-                elif not 'box' in str(results.key_list[i].object_type).lower():
-                    results[results.key_list[i]] = np.array(results.value_list[i])
-            return results
 
     def apply(self, input: Tuple, spec: TensorSpec):
         return super().apply(input, spec)
+
+    def _apply_image(self, image, spec: TensorSpec):
+        image = unpack_singleton(image)
+
+        if self._shape_info is None:
+            self._shape_info = self._get_shape(image)
+        keep,other_idxes,center,img1_offsetx,img1_offsety,img2_offsetx,img2_offsety,img3_offsetx,img3_offsety = self._shape_info
+        if keep:
+            return image
+        height,width=self.output_size
+        p1 = self.memory_cache[other_idxes[0]][spec].copy()[img1_offsety:img1_offsety+center[0], img1_offsetx:img1_offsetx+width-center[1], :]
+        p2 = self.memory_cache[other_idxes[1]][spec].copy()[img2_offsety:img2_offsety+height-center[0], img2_offsetx:img2_offsetx+center[1], :]
+        p3 = self.memory_cache[other_idxes[2]][spec].copy()[img3_offsety:img3_offsety+height-center[0], img3_offsetx:img3_offsetx+width-center[1], :]
+
+        base_img = image.copy()
+        base_img[:p1.shape[0], center[1]:center[1]+p1.shape[1], :] = p1
+        base_img[center[0]:center[0]+p2.shape[0], :p2.shape[1], :] = p2
+        base_img[center[0]:center[0]+p3.shape[0], center[1]:center[1]+p3.shape[1] :] = p3
+
+        return clip(image, 0, 255).astype(np.float32)
+
+
+    def _apply_coords(self, coords, spec: TensorSpec):
+        keep, other_idxes, center,img1_offsetx,img1_offsety,img2_offsetx,img2_offsety,img3_offsetx,img3_offsety = self._shape_info
+        if keep:
+            return coords
+        height,width=self.output_size
+        box0 =coords.copy()
+        box0[:, 0] = np.clip(box0[ :, 0], 0, center[1])
+        box0[:, 1] = np.clip(box0[:, 1], 0, center[0])
+
+        idxs = np.array([(0, 1), (2, 1), (0, 3), (2, 3)]).flatten()
+
+
+        box1 = self.memory_cache[other_idxes[0]][spec].copy()[:,:4]
+        box1 = np.asarray(box1).reshape(-1, 4)[:, idxs].reshape(-1, 2)
+            #.reshape(coords.shape[0],-1,2)
+        box1[:, 0] = np.clip(box1[:, 0], img1_offsetx,img1_offsetx+width-center[1])
+        box1[:, 1] = np.clip(box1[:, 1], img1_offsety,img1_offsety+center[0])
+
+
+        box2 = self.memory_cache[other_idxes[1]][spec].copy()[:,:4]
+        box2= np.asarray(box2).reshape(-1, 4)[:, idxs].reshape(-1, 2)
+        box2[:, 0] = np.clip(box2[:, 0], img2_offsetx,img2_offsetx+center[1])
+        box2[:, 1] = np.clip(box2[:, 1], img2_offsety,img2_offsety+height-center[0])
+
+
+        box3 = self.memory_cache[other_idxes[2]][spec].copy()[:,:4]
+        box3 = np.asarray(box3).reshape(-1, 4)[:, idxs].reshape(-1, 2)
+        box3[:, 0] = np.clip(box3[ :, 0], img3_offsetx,img3_offsetx+width-center[1])
+        box3[ :, 1] = np.clip(box3[:, 1], img3_offsety,img3_offsety+height-center[0])
+
+        box_list=np.concatenate([box0,box1,box2,box3],axis=0)
+        box_list=box_list.reshape(-1,4)
+        hw = np.clip(box_list[..., 2:] - box_list[..., :2], 0.0, None)
+        area=hw[..., 0] * hw[..., 1]
+        mask_box=area>0
+        mask_box=box_list[mask_box,:]
+        return mask_box
+
+
+
+    def _apply_mask(self, mask, spec: TensorSpec):
+        keep, other_idxes, center,img1_offsetx,img1_offsety,img2_offsetx,img2_offsety,img3_offsetx,img3_offsety = self._shape_info
+        height, width = self.output_size
+        if keep:
+            return mask
+        mp1 = self.memory_cache[other_idxes[0]][spec].copy()[img1_offsety:img1_offsety+center[0], img1_offsetx:img1_offsetx+width-center[1], :]
+        mp2 = self.memory_cache[other_idxes[1]][spec].copy()[img2_offsety:img2_offsety+height-center[0], img2_offsetx:img2_offsetx+center[1], :]
+        mp3 = self.memory_cache[other_idxes[2]][spec].copy()[img3_offsety:img3_offsety+height-center[0], img3_offsetx:img3_offsetx+width-center[1], :]
+
+        base_msk = mask.copy()
+        base_msk[:mp1.shape[0], center[1]:center[1] + mp1.shape[1], :] = mp1
+        base_msk[center[0]:center[0] + mp2.shape[0], :mp2.shape[1], :] = mp2
+        base_msk[center[0]:center[0] + mp3.shape[0], center[1]:center[1] + mp3.shape[1]:] = mp3
+        return base_msk
+
+
+    def _get_shape(self, image):
+        if self.output_size is None:
+            self.output_size=image.shape[:2]
+        rr = random.random()
+        keep=rr<self.keep_prob or len(self.memory_cache[:-1])<=3
+        other_idxes=None
+        center=None
+        if not keep:
+            other_idxes = list(range(len(self.memory_cache[:-1])))
+            random.shuffle(other_idxes)
+            other_idxes = other_idxes[:3]
+
+            height, width = self.output_size
+            rs = np.random.uniform(0.5, 1.5, [2])  # random shift
+            center = (int(height * rs[0] / 2), int(width * rs[1] / 2))
+
+            img1 = self.memory_cache[other_idxes[0]].value_list[0]
+            img1_offsetx=img1.shape[1] if img1.shape[1]-1<(width-center[1]) else np.random.choice(np.arange(0,img1.shape[1]-(width-center[1]),1))
+            img1_offsety = img1.shape[0] if img1.shape[0] - 1 < center[0] else np.random.choice(np.arange(0, img1.shape[0] - center[0], 1))
+
+            img2 = self.memory_cache[other_idxes[1]].value_list[0]
+            img2_offsetx = img2.shape[1] if img2.shape[1] - 1 < center[1] else np.random.choice(np.arange(0, img2.shape[1] - center[1], 1))
+            img2_offsety = img2.shape[0] if img2.shape[0] - 1 < (height-center[0])else np.random.choice(np.arange(0, img2.shape[0] - (height-center[0]), 1))
+
+
+            img3 = self.memory_cache[other_idxes[2]].value_list[0]
+            img3_offsetx = img3.shape[1] if img3.shape[1] - 1 < (width - center[1]) else np.random.choice(np.arange(0, img3.shape[1] - (width - center[1]), 1))
+            img3_offsety = img3.shape[0] if img3.shape[0] - 1 < (height-center[0]) else np.random.choice(np.arange(0, img3.shape[0] - (height-center[0]), 1))
+
+        return (keep,other_idxes,center,img1_offsetx,img1_offsety,img2_offsetx,img2_offsety,img3_offsetx,img3_offsety)
+
+# class ImageMosaic1(VisionTransform):
+#     """Perform gamma correction on an image.
+#         Also known as Power Law Transform. Intensities in RGB mode are adjusted
+#         based on the following equation:
+#         I_out = 255 * gain * ((I_in / 255) ** gamma)
+#         See https://en.wikipedia.org/wiki/Gamma_correction for more details.
+#     Args:
+#         gamma (float): Non negative real number. gamma larger than 1 make the
+#             shadows darker, while gamma smaller than 1 make dark regions
+#             lighter.
+#         gain (float): The constant multiplier.
+#     """
+#
+#     def __init__(self, output_size=None, keep_prob=0.7, name='image_mosaic', **kwargs):
+#         super().__init__(name)
+#         self.output_size = output_size
+#         self.keep_prob = keep_prob
+#
+#     def apply_batch(self, inputs: Sequence[Tuple], spec: Optional[TensorSpec] = None):
+#         r"""Apply transform on batch input data."""
+#         if not isinstance(inputs, OrderedDict):
+#             if spec is None and self.is_spatial == True:
+#                 self._shape_info = None
+#                 spec = TensorSpec(shape=tensor_to_shape(inputs, need_exclude_batch_axis=True, is_singleton=True), object_type=ObjectType.rgb)
+#             return self.apply(inputs, spec)
+#         else:
+#             results = OrderedDict()
+#             imgs = None
+#             boxes = None
+#             masks = None
+#             imgs = inputs.value_list[0]
+#
+#             if int_shape(imgs)[0] <= 4:
+#                 return inputs
+#             else:
+#                 for i in range(len(inputs.values())):
+#                     k, v = inputs.item_list[i]
+#                     results[k] = []
+#                     if i == 0:
+#                         imgs = v
+#                     elif isinstance(k, TensorSpec) and 'box' in str(k.object_type).lower():
+#                         boxes = v
+#                     elif isinstance(k, TensorSpec) and 'mask' in str(k.object_type).lower():
+#                         masks = v
+#                 if get_backend() == 'pytorch':
+#                     imgs = imgs.transpose([0, 2, 3, 1])
+#
+#                 for i in range(int_shape(imgs)[0]):
+#                     rr = random.random()
+#                     if rr > self.keep_prob:
+#
+#                         img = imgs[i]
+#
+#                         other_idxes = list(range(int_shape(imgs)[0]))
+#                         other_idxes.remove(i)
+#                         random.shuffle(other_idxes)
+#                         other_idxes = other_idxes[:3]
+#
+#                         height, width = self.output_size
+#                         rs = np.random.uniform(0.5, 1.5, [2])  # random shift
+#                         center = (int(height * rs[0] / 2), int(width * rs[1] / 2))
+#
+#                         # crop each image
+#                         p1 = imgs[other_idxes[0]].copy()[:center[0], center[1]:width, :]
+#                         p2 = imgs[other_idxes[1]].copy()[center[0]:height, :center[1], :]
+#                         p3 = imgs[other_idxes[2]].copy()[center[0]:height, center[1]:width, :]
+#
+#                         base_img = img.copy()
+#                         base_img[:center[0], center[1]:width, :] = p1
+#                         base_img[center[0]:height, :center[1], :] = p2
+#                         base_img[center[0]:height, center[1]:width, :] = p3
+#
+#                         results[results.key_list[0]].append(base_img)
+#
+#                         if masks is not None and masks[i].any():
+#                             msk = masks[i]
+#                             mp1 = masks[other_idxes[0]].copy()[:center[0], center[1]:width]
+#                             mp2 = masks[other_idxes[1]].copy()[center[0]:height, :center[1]]
+#                             mp3 = masks[other_idxes[2]].copy()[center[0]:height, center[1]:width]
+#
+#                             base_msk = msk.copy()
+#                             base_msk[:center[0], center[1]:width] = mp1
+#                             base_msk[center[0]:height, :center[1]] = mp2
+#                             base_msk[center[0]:height, center[1]:width] = mp3
+#
+#                             mask_key = [k for k in results.key_list if 'mask' in str(k.object_type).lower()]
+#                             if mask_key:
+#                                 results[mask_key[0]].append(base_msk)
+#
+#                         mosaic_bboxes = []
+#                         if boxes is not None and boxes[i].any():
+#                             box = boxes[i]
+#                             box1 = boxes[other_idxes[0]]
+#                             box2 = boxes[other_idxes[1]]
+#                             box3 = boxes[other_idxes[2]]
+#                             box_list = []
+#                             if box.any():
+#                                 box[:, 0][box[:, 0] > center[1]] = center[1]
+#                                 box[:, 2][box[:, 2] > center[1]] = center[1]
+#                                 box[:, 1][box[:, 1] > center[0]] = center[0]
+#                                 box[:, 3][box[:, 3] > center[0]] = center[0]
+#                                 box_list.append(box)
+#                             if box1.any():
+#                                 box1[:, 0][box1[:, 0] < center[1]] = center[1]
+#                                 box1[:, 2][box1[:, 2] < center[1]] = center[1]
+#                                 box1[:, 1][box1[:, 1] > center[0]] = center[0]
+#                                 box1[:, 3][box1[:, 3] > center[0]] = center[0]
+#                                 box_list.append(box1)
+#
+#                             if box2.any():
+#                                 box2[:, 0][box2[:, 0] > center[1]] = center[1]
+#                                 box2[:, 2][box2[:, 2] > center[1]] = center[1]
+#                                 box2[:, 1][box2[:, 1] < center[0]] = center[0]
+#                                 box2[:, 3][box2[:, 3] < center[0]] = center[0]
+#                                 box_list.append(box2)
+#
+#                             if box3.any():
+#                                 box3[:, 0][box3[:, 0] < center[1]] = center[1]
+#                                 box3[:, 2][box3[:, 2] < center[1]] = center[1]
+#                                 box3[:, 1][box3[:, 1] < center[0]] = center[0]
+#                                 box3[:, 3][box3[:, 3] < center[0]] = center[0]
+#                                 box_list.append(box3)
+#
+#                             if len(box_list) > 0:
+#                                 mosaic_bboxes = np.concatenate(box_list)
+#                                 # remove no area bbox
+#                                 keep_x = mosaic_bboxes[:, 2] - mosaic_bboxes[:, 0] > 16
+#                                 keep_y = mosaic_bboxes[:, 3] - mosaic_bboxes[:, 1] > 16
+#                                 keep_mask = np.logical_and(keep_x, keep_y)
+#                                 mosaic_bboxes = mosaic_bboxes[keep_mask]
+#                             else:
+#                                 mosaic_bboxes = list()
+#                         box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
+#                         if box_key:
+#                             results[box_key[0]].append(mosaic_bboxes)
+#                     else:
+#                         results[results.key_list[0]].append(imgs[i])
+#                         mask_key = [k for k in results.key_list if 'mask' in str(k.object_type).lower()]
+#                         if mask_key:
+#                             results[mask_key[0]].append(masks[i])
+#                         box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
+#                         if box_key:
+#                             results[box_key[0]].append(boxes[i])
+#
+#             for i in range(len(results)):
+#                 if i == 0:
+#                     result_imgs = np.stack(results.value_list[i], 0)
+#                     if get_backend() == 'pytorch':
+#                         result_imgs = result_imgs.transpose([0, 3, 1, 2])
+#
+#                     results[results.key_list[0]] = result_imgs
+#                 elif not 'box' in str(results.key_list[i].object_type).lower():
+#                     results[results.key_list[i]] = np.array(results.value_list[i])
+#             return results
+#
+#     def apply(self, input: Tuple, spec: TensorSpec):
+#         return super().apply(input, spec)
