@@ -259,8 +259,6 @@ class YoloLayer(Layer):
 
 
         prediction = x.view(num_batch, self.num_anchors, self.num_classes + 5, grid_size, grid_size).permute(0, 1, 3, 4, 2).contiguous()
-        # if self.training:
-        #     return reshape(prediction,(num_batch, -1, self.num_classes + 5))
 
             # Get outputs
         xy = sigmoid(prediction[..., 0:2])  # Center x
@@ -279,10 +277,10 @@ class YoloLayer(Layer):
         if self.small_item_enhance and self.stride == 8:
             pred_conf = (pred_conf * cls_probs).sqrt()
 
-        output = torch.cat([xy, wh, pred_conf, pred_class], -1)
+        return  torch.cat([xy, wh, pred_conf, pred_class], -1)
+
 
 
-        return output
 
 
 
