@@ -326,7 +326,7 @@ class _ClassificationLoss(Loss):
             is_hard = None
             if target.dtype == Dtype.int64:
                 is_hard = greater(target, 0)
-                num_hard = is_hard.sum().item()
+                num_hard =builtins.max( is_hard.sum().item(),1)
                 num_easy = int(self.ohem_ratio * num_hard)
             hard_cases = is_hard > 0
 
@@ -892,12 +892,12 @@ class FocalLoss(_ClassificationLoss):
         Returns:
 
             """
-
+        if self.is_logsoftmax:
+            output = clip(exp(output), 1e-8, 1 - 1e-8)
+            self.from_logits = True
         if not self.need_target_onehot:
             if self.is_target_onehot:
                 target = argmax(target, self.axis)
-            if self.is_logsoftmax == False:
-                output = torch.log_softmax(output, self.axis)
             probs = exp(output)
             return F.nll_loss(pow(1 - probs, self.gamma) * output, target, self.sample_weight, ignore_index=self.ignore_index, reduction='none')
         else:
@@ -912,8 +912,7 @@ class FocalLoss(_ClassificationLoss):
                 if self.is_logsoftmax == True:
                     sample_weight = self.sample_weight.view(*reshape_shape) * self.ignore_index_weight.view(*reshape_shape)
 
-            if self.is_logsoftmax == False:
-                output = torch.log_softmax(output, self.axis)
+
             probs = exp(output) * sample_weight
             loss = -self.alpha * target * pow((1 - probs), self.gamma) * output
             return loss
