@@ -68,6 +68,7 @@ class TrainingPlan(object):
         self.num_epochs = 1
         self._batch_size = 1
         self.steps = 0
+        self.epochs = 0
         self.warmup = 0
         self.default_collect_data_inteval = 1
         self.print_progress_frequency = 10
@@ -395,6 +396,7 @@ class TrainingPlan(object):
                     data_feed['x'] = 'input'
                     available_items.remove('input')
 
+
             if len(trainingitem.signature.inputs) == len(data_symbols) == 1:
                 # if trainingitem.signature.inputs.value_list[0].shape.is_compatible_with(data_provider.traindata.data.element_spec.shape):
                 data_feed[trainingitem.signature.inputs.key_list[0]] = data_provider.traindata.data.symbol
@@ -414,6 +416,11 @@ class TrainingPlan(object):
                     data_feed[out] = out
                     available_items.remove(out)
 
+            if 'target' in data_feed:
+                if len(label_symbols) == 1 and label_symbols[0] in available_items:
+                    data_feed['target'] = label_symbols[0]
+                    available_items.remove(label_symbols[0])
+
             for key in data_feed.keys():
                 if data_feed[key] == None and key in available_items:
                     data_feed[key] = key
@@ -468,9 +475,9 @@ class TrainingPlan(object):
                 t1.start()
                 open_browser('http://{0}:{1}/'.format(ctx.tensorboard_server, ctx.tensorboard_port), 5)
             if ctx.enable_mlflow:
-                t1 = threading.Thread(target=launchMLFlow, args=([]))
-                t1.setDaemon(True)
-                t1.start()
+                t2 = threading.Thread(target=launchMLFlow, args=([]))
+                t2.setDaemon(True)
+                t2.start()
                 ctx.mlflow_logger.start_run()
                 open_browser('http://{0}:{1}/'.format(ctx.mlflow_server, ctx.mlflow_port), 5)
             if not is_resume or only_steps == True:
@@ -505,6 +512,7 @@ class TrainingPlan(object):
                 data_provider.traindata.batch_sampler._batch_transform_funcs = data_provider._batch_transform_funcs
             self.do_on_training_start()
             for epoch in range(self.num_epochs):
+                self.epochs=epoch
                 try:
                     for mbs, return_data in enumerate(data_provider):
 
